#!/usr/bin/perl
use strict;
use List::MoreUtils qw(uniq);
use List::Util qw(min max);
##########################################################################################
sub representative_genotype
{
  #The maximum occuring genotype will be the representative genotype. cutoff will decide what should be minimum frequency of the genotype e.g. 80% of 4 samples means 3 samples should have the representative genotype. if there is a tie, then both genotypes will be output but this will reduce its chance of popping up into the breed specific markers.
  my $geno = $_[0];
  my $rep_geno;
  my $rep_geno_count=0;
  my %count;
  foreach my $word(@$geno)  
  { 
    $count{$word}++; 
  }
  foreach my $word (sort keys %count)
  { 
    if($count{$word}>$rep_geno_count)
	{
		$rep_geno=$word;
		$rep_geno_count=$count{$word};
	}
    #print $word, ":", $count{$word}, "\t"; 
  } 
  return $rep_geno;
}
##########################################################################################
# (1) quit unless we have the correct number of command-line args
my $num_args = $#ARGV + 1;

if ($num_args != 2) 
{
    print "\n Insufficient input files\n1: input Breed File \n2: Search Target file\n";
    exit;
}
my $query_file="";
$query_file=$ARGV[0];
my @temp=split /\./, $query_file;
my $query_filename=$temp[0];
my $search_file="";
$search_file=$ARGV[1];

my $res;
my $search_id="";
my @query_ltr=();
my @search_ltr=();
my @union=();
my @isect=();
my $query_line=""; 
my $search_line=""; 
open(QUERYFILE, "<$query_file") or die "Couldn't open file 1, $!";
open(SEARCHFILE, "<$search_file") or die "Couldn't open file 2, $!";
open(OUT,'>',$query_filename."_specific.vcf")  or die $!;
##########################################################################################
my $i=0;

while(($query_line=<QUERYFILE>) and ($search_line=<SEARCHFILE>))
{
	chomp($query_line);
	chomp($search_line);
		
	if($query_line =~/^#/)
	{
		print OUT $query_line."\n";
	}else{
		#print ($query_line."\n");
		#print ($search_line."\n");
		@query_ltr=split /\t/, $query_line;
		@search_ltr=split /\t/, $search_line;		
		my $chrom=$query_ltr[0];
		my $position=$query_ltr[1];
		my $ref_allele=$query_ltr[3];
		my $alt_allele=$query_ltr[4];
		my $no_of_query_cols= scalar(@query_ltr);
		my @query_genotypes=@query_ltr[9..$#query_ltr];
		my @search_genotypes=@search_ltr[9..$#search_ltr];
		my @query_genotypes_uniq=uniq(@query_genotypes);
		my @search_genotypes_uniq=uniq(@search_genotypes);
		#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		my @qr=();
		foreach (@query_genotypes) 
		{
			my @separated = split(':', $_);
			if($separated[0] ne "./.")
			{
				push @qr, $separated[0];
				#print $_."===>".$separated[0]."\t";
			}
		}
		#print scalar @qr."\n";
		#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		my @sr=();
		foreach (@search_genotypes) 
		{
			my @separated = split(':', $_);
			if($separated[0] ne "./.")
			{
				push @sr, $separated[0];
			}
		}		
		#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
		my $res=representative_genotype(\@qr);
		my @rep_geno=($res);
		#find the intersection between two arrays	
		my @intersection =
			grep { defined }
				@{ { map { lc ,=> $_ } @rep_geno } }
				   { map { lc } uniq(@sr) };

		my $no_of_common_genotypes= scalar @intersection;
		if($no_of_common_genotypes==0)
		{
			#my $output_line=$chrom."\t".$position."\t.\t".$ref_allele."\t".$alt_allele."\t.\t.\tPR\tGT\t\n";
			my $output_line=$query_line."\n";
			print OUT $output_line;
		}
			
	}
	
	print "\n";
}

close(QUERYFILE) || die "Couldn't close file properly";
close(SEARCHFILE) || die "Couldn't close file properly";
close(OUT)